/* memory.js - TyranoScript translation memory class
 * Copyright (C) 2016 Jaypee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
(function (memory) {
  memory.init = function (json, priority) {
    this.json = json;
    this.dict = {};
    this.priority = priority;
    for (var prop in json) {
      this.updateDict(prop);
    }
    this.tags = {};
    this.method = 'default';
    this.prune = false;
  };

  memory.setUpdateLangs = function (tags) {
    tags.forEach(function (tag) {
      this[tag] = null;
    }, this.tags);
  };

  memory.setUpdateMethod = function (method) {
    this.method = method;
  }

  memory.setUpdatePrune = function (prune) {
    this.prune = prune;
  }

  memory.lookup = function (prop, key) {
    var entry = this.getDict(prop)[key];
    if (entry) {
      for (var i = 0; i < this.priority.length; i++) {
        if (entry[this.priority[i]])
          return entry[this.priority[i]];
      }
    }
  };

  memory.updateDict = function (prop) {
    this.getJSON(prop).forEach(function add(entry) {
      this[entry.key] = entry;
    }, this.dict[prop] = {});
  };

  memory.getJSON = function (prop) {
    return (this.json[prop] = this.json[prop] || []);
  }

  memory.getDict = function (prop) {
    return (this.dict[prop] = this.dict[prop] || {});
  }

  memory.add = function (prop, keys) {
    var json = this.getJSON(prop), dict = this.getDict(prop);

    var entries = keys.map(function (key) {
      return dict[key] || $.extend({ key: key }, this.tags);
    }, this);
    if (this.method === 'default') {
      this.json[prop] = this.prune ? entries : this.merge(entries, json);
    } else {
      if (this.prune) {
        json = json.filter(function prune(entry) {
          return this.has(entry);
        }, new Set(entries));
      }
      entries = entries.filter(function fresh(entry) {
        return typeof dict[entry.key] === 'undefined';
      });
      if (this.method === 'append') {
        this.json[prop] = json.concat(entries);
      } else {
        this.json[prop] = entries.concat(json);
      }
    }
    this.updateDict(prop);
  };

})(window.TranslationMemory = window.TranslationMemory || {});
