/* merge.js - Topological merge for translation memory
 * Copyright (C) 2016 Jaypee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
(function (memory) {

  function merge(a, b) {
    var nodes = new Map();
    var list = [];
    var queue = [];
    var prev = null;

    a.forEach(function (value, index, array) {
      var node = new this.Node(value, index, true);
      if (index) {
        prev.createLink(node);
      } else {
        queue.push(node);
      }
      nodes.set(value, node);
      prev = node;
    }, merge);

    var prev_b = null;
    b.forEach(function (value, index, array) {
      var node = nodes.get(value);
      if (!node) {
        var node = new this.Node(value);
        if (index) {
          prev.createLink(node);
          node.index = prev.index;
        } else {
          queue.unshift(node);
        }
        nodes.set(value, node);
        node.prev_b = prev_b;
        prev_b = node;
      } else if (index) {
        var pred = prev_b;
        while (pred && pred.index > node.index) {
          pred = pred.prev_b;
        }
        if (pred) {
          pred.createLink(node);
          if (queue[queue.length - 1] === node) {
            queue.pop();
          }
        }
      }
      prev = node;
    }, merge);

    while (queue.length !== 0) {
      var node = queue.pop();
      list.push(node.value);
      node.out.forEach(function (next) {
        node.deleteLink(next);
        if (!next.in.size) {
          if (next.priority) {
            queue.push(next);
          } else {
            queue.unshift(next);
          }
        }
      });
    }

    return list;
  };

  merge.Node = function (value, index, priority) {
    this.value = value;
    if (typeof index !== 'undefined') {
      this.index = index;
    } else {
      this.index = -1;
    }
    if (typeof priority !== 'undefined') {
      this.priority = priority;
    } else {
      this.priority = false;
    }
    this.in = new Set();
    this.out = new Set();
  };

  merge.Node.prototype.createLink = function (node) {
    this.out.add(node);
    node.in.add(this);
  };

  merge.Node.prototype.deleteLink = function (node) {
    this.out.delete(node);
    node.in.delete(this);
  };

  memory.merge = merge;

})(window.TranslationMemory = window.TranslationMemory || {});
