/* reduce.js - Utility for reducing tags in TyranoScript scenarios
 * Copyright (C) 2016 Jaypee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
window.TyranoTranslate = window.TyranoTranslate || {};
(function (reduce) {
  reduce.UNIT_TAGS = {
    'text': true, 'emb': true, 'p': true, 'l': true, 'r': true, 'lr': true,
  };

  reduce.BLOCK_TAGS = {
    'html': 'endhtml', 'iscript': 'endscript'
  };

  reduce.parseScene = function (tags) {
    var state = {
      start: null,
      units: { text: [], script: [], html: [] },
      blocks: []
    };
    tags.reduce(this.reduceScene.bind(this), state);
    return state.units;
  };

  reduce.reduceScene = function (state, tag, i, scene) {
    if (this.isUnitTag(tag)) {
      if (state.start === null) {
        state.start = i;
      } else if (tag.line !== scene[i - 1].line) {
        this.addUnitHelper(state, scene, i);
        state.start = i;
      }
    } else if (state.start !== null) {
      this.addUnitHelper(state, scene, i);
      state.start = null;
    }
    this.blockTagsHelper(state.blocks, tag.name);
    return state;
  };

  reduce.addUnitHelper = function (state, scene, i) {
    var tags = scene.slice(state.start, i);
    var unit = {
      tags: tags,
      start: state.start,
      end: i,
      length: i - state.start,
      source: this.unitToString(tags)
    };
    if (state.blocks.length === 0) {
      state.units.text.push(unit);
    } else if (state.blocks[state.blocks.length - 1] === 'endscript') {
      state.units.script.push(unit);
    } else if (state.blocks[state.blocks.length - 1] === 'endhtml') {
      state.units.html.push(unit);
    }
  };

  reduce.blockTagsHelper = function (blocks, name) {
    if (this.BLOCK_TAGS.hasOwnProperty(name)) {
      blocks.push(this.BLOCK_TAGS[name]);
    } else if (name === blocks[blocks.length - 1]) {
      blocks.pop();
    }
  };

  reduce.unitToString = function (tags) {
    return tags.reduce(this.reduceUnit.bind(this), '');
  };

  reduce.reduceUnit = function (text, currentTag) {
    return currentTag.name === 'text'
      ? text + currentTag.pm.val
      : text + this.tagToString(currentTag);
  };

  reduce.tagToString = function (tag) {
    var params = '';
    for (key in tag.pm) {
      params += ' ' + key + "='" + tag.pm[key] + "' ";
    }
    return '[' + tag.name + params + ']';
  };

  reduce.isUnitTag = function (tag) {
    return this.UNIT_TAGS[tag.name];
  };

  reduce.addUnitTags = function (names) {
    names.forEach(function (name) {
      this.UNIT_TAGS[name] = true;
    }, this);
  };

})(TyranoTranslate.reduce = TyranoTranslate.reduce || {});
