/* translate.js - TyranoScript translation plugin hooks
 * Copyright (C) 2016 Jaypee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
window.TyranoTranslate = window.TyranoTranslate || {};
(function (translate) {
  translate.init = function (detour, reduce, memory) {
    this.parser = Object.create(tyrano.plugin.kag.parser);
    this.memory = memory;
    this.registerHooks(detour, reduce);
  };

  translate.registerHooks = function (detour, reduce) {
    detour.attachParseScenario();
    detour.addHookParseScenario(this.parseScenarioHook.bind(this, reduce));

    detour.attachTagStart('button');
    detour.addHookTagStart('button', this.graphicHook.bind(this));
    detour.addHookTagStart('button', this.hintHook.bind(this));

    detour.attachTagStart('chara_ptext');
    detour.addHookTagStart('chara_ptext', this.charaHook.bind(this));

    detour.attachTagStart('eval');
    detour.addHookTagStart('eval', this.evalHook.bind(this));

    detour.attachTagStart('if');
    detour.addHookTagStart('if', this.branchHook.bind(this));
    detour.attachTagStart('elsif');
    detour.addHookTagStart('elsif', this.branchHook.bind(this));
  };

  translate.graphicHook = function (pm) {
    pm.hint = pm.hint || this.memory.lookup('graphic', pm.graphic);
  };

  translate.hintHook = function (pm) {
    pm.hint = this.memory.lookup('hint', pm.hint) || pm.hint;
  };

  translate.charaHook = function (pm) {
    pm.name = this.memory.lookup('chara', pm.name) || pm.name;
  };

  translate.evalHook = function (pm) {
    pm.exp = this.memory.lookup('eval', pm.exp) || pm.exp;
  };

  translate.branchHook = function (pm) {
    pm.exp = this.memory.lookup('branch', pm.exp) || pm.exp;
  };

  translate.parseScenarioHook = function (reduce, result_obj) {
    var array_tags = result_obj.array_s;
    var units = reduce.parseScene(array_tags);

    var replacements = units.text.filter(function (unit) {
      var text = this.memory.lookup('text', unit.source);
      if (text) {
        unit.replacement = this.parser.parseScenario(text).array_s;
      }
      return (typeof unit.replacement !== 'undefined');
    }, this);

    var offset = 0;
    replacements.forEach(function (unit) {
      var args = [unit.start + offset, unit.length];
      var line = unit.tags[0].line;
      args = args.concat(cloneUnit(unit, line));
      array_tags.splice.apply(array_tags, args);
      offset += unit.replacement.length - unit.length;
    }, this);

    return result_obj;
  };

  var cloneUnit = function (unit, line) {
    return unit.replacement.reduce(function (array_s, tag) {
      var copy = $.extend({}, tag);
      copy.line = line + copy.line;
      array_s.push(copy);
      return array_s;
    }, []);
  };

})(TyranoTranslate.translate = TyranoTranslate.translate || {});
