/* update.js - Utility for updating a TyranoScript translation
 * Copyright (C) 2016 Jaypee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
try {
window.TyranoTranslate = window.TyranoTranslate || {};
(function (update) {

  update.getLines = function (scenes) {
    var lines = { text: [], script: [], html: [] };
    scenes.forEach(function (scene) {
      var units = this.reduce.parseScene(scene.array_s);
      for (var type in lines) {
        var strings = units[type].map(function (unit) {
          return unit.source;
        });
        lines[type] = lines[type].concat(strings);
      }
    }, this);
    return lines;
  };

  function unique(array) {
    return array.filter(function (element) {
      return element in this ? false : this[element] = true;
    }, {});
  }

  update.parseSceneFile = function (filename) {
    var text = fs.readFileSync(filename, 'utf-8');
    return this.kag.parser.parseScenario(text);
  };

  update.writeDict = function () {
    var scenes = [];
    this.util.getScenes().forEach(function (file) {
      scenes.push(this.parseSceneFile(file));
    }, this);
    var lines = this.getLines(scenes);
    for (var type in lines) {
      if (this.config.props.indexOf(type) != -1) {
        this.memory.add(type, unique(lines[type]));
      }
    }
    if (this.config.props.indexOf('hint') != -1) {
      var hints = [];
      scenes.forEach(function (scene) {
        scene.array_s.forEach(function (tag) {
          if (tag.name === 'button' && tag.pm.hint) {
            hints.push(tag.pm.hint);
          }
        });
      });
      this.memory.add('hint', unique(hints));
    }
    if (this.config.props.indexOf('chara') != -1) {
      var charas = [];
      scenes.forEach(function (scene) {
        scene.array_s.forEach(function (tag) {
          if (tag.name === 'chara_ptext' && tag.pm.name) {
            charas.push(tag.pm.name);
          }
        });
      });
      this.memory.add('chara', unique(charas));
    }
    if (this.config.props.indexOf('graphic') != -1) {
      var graphics = [];
      scenes.forEach(function (scene) {
        scene.array_s.forEach(function (tag) {
          if (tag.name === 'button' && tag.pm.graphic) {
            graphics.push(tag.pm.graphic);
          }
        });
      });
      this.memory.add('graphic', unique(graphics));
    }
    if (this.config.props.indexOf('eval') != -1) {
      var evals = [];
      scenes.forEach(function (scene) {
        scene.array_s.forEach(function (tag) {
          if (tag.name === 'eval' && tag.pm.exp) {
            evals.push(tag.pm.exp);
          }
        });
      });
      this.memory.add('eval', unique(evals));
    }
    if (this.config.props.indexOf('branch') != -1) {
      var branches = [];
      scenes.forEach(function (scene) {
        scene.array_s.forEach(function (tag) {
          if ((tag.name === 'if' || tag.name === 'elsif') && tag.pm.exp) {
            branches.push(tag.pm.exp);
          }
        });
      });
      this.memory.add('branch', unique(branches));
    }
    var filename = path.join(this.util.assetDir, this.config.file);
    var text = JSON.stringify(this.memory.json, null, 2);
    fs.writeFileSync(filename, text);
  };

  update.init = function (kag, reduce, memory, config) {
    this.kag = kag;
    this.reduce = reduce;
    this.memory = memory;
    this.config = config;
    this.util = Object.create(TyranoTranslate.util);
    this.util.init();
  };

  var fs = require('fs');
  var path = require('path');

})(TyranoTranslate.update = TyranoTranslate.update || {});
} catch (e) { console.error(e); }
