/* util.js - Utility methods for TyranoScript translation plugin
 * Copyright (C) 2016 Jaypee
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */
window.TyranoTranslate = window.TyranoTranslate || {};
(function (util) {
  function walk(memo, dir) {
    var items = getDirectoryListing(dir);
    memo = memo.concat(items.files);
    return items.dirs.reduce(walk, memo);
  };

  function getDirectoryListing(dir) {
    var items = fs.readdirSync(dir);
    items = items.map(function (name) {
      return path.join(dir, name);
    });
    var list = { files: [], dirs: [] };
    return items.reduce(toType, list);
  };

  function toType(memo, pathname) {
    var stat = fs.lstatSync(pathname);
    if (!stat.isSymbolicLink()) {
      if (stat.isFile()) {
        memo.files.push(pathname);
      } else if (stat.isDirectory()) {
        memo.dirs.push(pathname);
      }
    }
    return memo;
  };

  function getFileContents(memo, name) {
    memo[name] = fs.readFileSync(name, 'utf-8');
    return memo;
  };

  util.getScenes = function () {
    var files = walk([], this.sceneDir);
    return files.filter(function (name) {
      return path.extname(name) === '.ks';
    });
  };

  util.init = function () {
    var pathname = window.location.pathname;
    var app_index = path.normalize(pathname.slice(1));
    var app_dir = path.dirname(app_index);
    this.sceneDir = path.join(app_dir, 'data', 'scenario');
    this.assetDir = path.join(app_dir, 'data', 'others', 'translate', 'assets');
  };

  var fs = require('fs');
  var path = require('path');
})(TyranoTranslate.util = TyranoTranslate.util || {});
